package de.fraunhofer.iem.dataprovider.configuration

import jakarta.annotation.PostConstruct
import org.springframework.beans.factory.annotation.Value
import org.springframework.boot.context.properties.source.InvalidConfigurationPropertyValueException
import org.springframework.stereotype.Component
import java.nio.file.Path
import java.nio.file.Paths
import kotlin.io.path.createDirectories

@Component
class DirectoryPathsConfig {
    @PostConstruct
    fun postConstruct() {
        // Creating the directories right after reading the values is the
        // easiest way to ensure that we have the correct read / write
        // permissions for the paths and fail as early if possible.
        createDir(GIT_CLONE_TARGET_DIRECTORY, gitCloneTargetDirectory)
        createDir(TOOL_RESULTS_TARGET_DIRECTORY, toolResultsTargetDirectory)
    }

    @Value("\${$GIT_CLONE_TARGET_DIRECTORY}")
    lateinit var gitCloneTargetDirectory: String

    @Value("\${$TOOL_RESULTS_TARGET_DIRECTORY}")
    lateinit var toolResultsTargetDirectory: String

    private fun createDir(variableName: String, stringPath: String) {
        if (stringPath.isBlank()) {
            throw InvalidConfigurationPropertyValueException(
                variableName,
                stringPath,
                "$variableName environment variable is not set."
            )
        } else {
            // There is no try catch block around the path operations on purpose!
            // We want to throw here if some of these operations fail.
            val path: Path = Paths.get(stringPath)
            path.createDirectories()
        }
    }

    companion object {
        private const val GIT_CLONE_TARGET_DIRECTORY = "GIT_CLONE_TARGET_DIRECTORY"
        private const val TOOL_RESULTS_TARGET_DIRECTORY = "TOOL_RESULTS_TARGET_DIRECTORY"
    }
}
